diff -rc stunnel-4.05/src/ssl.c stunnel-4.05.new/src/ssl.c
*** stunnel-4.05/src/ssl.c	2004-01-25 19:26:03.000000000 +0000
--- stunnel-4.05.new/src/ssl.c	2004-11-17 18:29:19.000000000 +0000
***************
*** 63,68 ****
--- 63,69 ----
  #endif
  static void print_stats(void);
  static void sslerror_stack(void);
+ static STACK_OF(X509_NAME) *get_CAList(const char *ca_file, const char *ca_path);
  
  SSL_CTX *ctx; /* global SSL context */
  static X509_STORE *revocation_store=NULL;
***************
*** 382,387 ****
--- 383,389 ----
  
  static void verify_init(void) {
      X509_LOOKUP *lookup;
+     STACK_OF(X509_NAME) *ca_list;
  
      if(options.verify_level<0)
          return; /* No certificate verification */
***************
*** 416,421 ****
--- 418,432 ----
          }
          log(LOG_DEBUG, "Verify directory set to %s", options.ca_dir);
      }
+         
+     ca_list = get_CAList(options.ca_file, options.ca_dir);
+         
+     if(!ca_list) {
+     	log(LOG_WARNING, "Warning: Unable to determine CA certs for"
+         	"client authentication");
+     } else {
+ 	SSL_CTX_set_client_CA_list(ctx, (STACK *)ca_list);
+     }
  
      if(options.crl_file || options.crl_dir) { /* setup CRL store */
          revocation_store=X509_STORE_new();
***************
*** 464,469 ****
--- 475,507 ----
          log(LOG_NOTICE, "Peer certificate location %s", options.ca_dir);
  }
  
+ static int xname_cmp(X509_NAME **a, X509_NAME **b)
+ {
+     return(X509_NAME_cmp(*a, *b));
+ }
+ 
+ static STACK_OF(X509_NAME) *get_CAList(const char *ca_file, const char *ca_path)
+ {
+     STACK_OF(X509_NAME) *ca_list;
+ 
+     /*Empty list*/
+     ca_list = sk_X509_NAME_new(xname_cmp);
+ 
+     /*Add CAFile*/
+     if (ca_file) {
+ 	SSL_add_file_cert_subjects_to_stack(ca_list, ca_file);
+     }
+ 
+     /*Add certs in CAPath*/
+     if (ca_path) {
+     	SSL_add_dir_cert_subjects_to_stack(ca_list, ca_path);
+     }
+ 
+     sk_X509_NAME_set_cmp_func(ca_list, NULL);
+ 
+     return ca_list;
+ }
+ 
  static int verify_callback(int preverify_ok, X509_STORE_CTX *callback_ctx) {
          /* our verify callback function */
      char txt[STRLEN];
